@@ -3,7 +3,6 @@ context("applications")
 
 source("utils.R")
 
-
 test_succeeds("keras pre-built models can be instantiated", {
   
   skip <- is.na(Sys.getenv("KERAS_TEST_APPLICATIONS", unset = NA)) && 
@@ -35,5 +34,10 @@ test_succeeds("keras pre-built models can be instantiated", {
   application_inception_v3()
 })
 
+test_succeeds("can use any input_shape", {
+  x <- application_vgg16(weights = NULL, input_shape = shape(NULL, NULL, 3), include_top = FALSE)
+  expect_s3_class(x, "keras.engine.training.Model")
+})
+
 
 
