@@ -61,7 +61,7 @@ test_call_succeeds("with_custom_object_scope", {
     data <- matrix(rexp(1000*784), nrow = 1000, ncol = 784)
     labels <- matrix(round(runif(1000*10, min = 0, max = 9)), nrow = 1000, ncol = 10)
      
-    model %>% fit(data, labels, epochs = 2)
+    model %>% fit(data, labels, epochs = 2, verbose = 0)
     
   })
   
